import time
import whatap.pack.tagcount_pack as tagcount_pack 
import whatap.osinfo as osinfo
import whatap.agent.secure.security_master as secure
from whatap.util.hash_util import HashUtil as hashutil
from whatap.value.list_value import ListValue 

class TaskInfraNicPerf:
    NIC = "infra_nic_perf"

    NIC_NAME = "name"
    NIC_TOTAL_BITPS = "totalbps"
    NIC_RX_BPS = "rxbps"
    NIC_TX_BPS = "txbps"
    NIC_TOTAL_PPS = "totalpps"
    NIC_RX_PPS = "rxpps"
    NIC_TX_PPS = "txpps"
    NIC_COLLISION = "collision"
    NIC_ERROR_IN = "errIn"
    NIC_ERROR_OUT = "errOut"
    NIC_DROP_IN = "dropIn"
    NIC_DROP_OUT = "dropOut"

    def __init__(self, interval = 5):
        self.intervalValue = interval
        self.recType = 2

    def interval(self):
        return self.intervalValue

    def process(self, os_, pcode, oid, now):
        netperfs = osinfo.GetNicUtil()
        if not netperfs:
            return None

        secu = secure.GetSecurityMaster()
        p = tagcount_pack.TagCountPack()
        p.pcode = pcode
        p.oid = oid
        p.time = now
        p.category = TaskInfraNicPerf.NIC
        p.putTag("oname", str(secu.oname))
        p.putTag("!rectype", str(self.recType))

        idList = ListValue()
        nameList = ListValue()
        totalbpsList = ListValue()
        rxbpsList = ListValue()
        txbpsList = ListValue()
        totalppsList = ListValue()
        rxppsList = ListValue()
        txppsList = ListValue()
        collisionList = ListValue()
        errorInList = ListValue()
        errorOutList = ListValue()
        dropInList = ListValue()
        dropOutList = ListValue()

        for netperf in netperfs:
            key = hashutil.hashFromString(netperf.desc)
            idList.addLong(key)
            nameList.addString(netperf.desc)
            totalbpsList.addFloat(netperf.trafficIn + netperf.trafficOut)
            rxbpsList.addFloat(netperf.trafficIn)
            txbpsList.addFloat(netperf.trafficOut)

            totalppsList.addFloat(netperf.packetIn + netperf.packetOut)
            rxppsList.addFloat(netperf.packetIn)
            txppsList.addFloat(netperf.packetOut)
            collisionList.addLong(0)
            errorInList.addFloat(netperf.errorIn)
            errorOutList.addFloat(netperf.errorOut)
            dropInList.addFloat(netperf.droppedIn)
            dropOutList.addFloat(netperf.droppedOut)


        p.put("@id", idList)
        p.put(TaskInfraNicPerf.NIC_NAME, nameList)
        p.put(TaskInfraNicPerf.NIC_TOTAL_BITPS, totalbpsList)
        p.put(TaskInfraNicPerf.NIC_RX_BPS, rxbpsList)
        p.put(TaskInfraNicPerf.NIC_TX_BPS, txbpsList)
        p.put(TaskInfraNicPerf.NIC_TOTAL_PPS, totalppsList)
        p.put(TaskInfraNicPerf.NIC_RX_PPS, rxppsList)
        p.put(TaskInfraNicPerf.NIC_TX_PPS, txppsList)
        p.put(TaskInfraNicPerf.NIC_COLLISION, collisionList)
        p.put(TaskInfraNicPerf.NIC_ERROR_IN, errorInList)
        p.put(TaskInfraNicPerf.NIC_ERROR_OUT, errorOutList)
        p.put(TaskInfraNicPerf.NIC_DROP_IN, dropInList)
        p.put(TaskInfraNicPerf.NIC_DROP_OUT, dropOutList)

        return p




